@@ -228,7 +228,7 @@ gar_shiny_getUrl <- function(session){
     url <- paste0(session$clientData$url_protocol,
                   "//",
                   hostname,
-                  if(!is.null(port)) paste0(":",port),
+                  if(port != "") paste0(":", port),
                   if(pathname != "/") pathname) 
     
     myMessage("Shiny URL detected as: ", url, level=1)
@@ -950,4 +950,4 @@ with_shiny <- function(f, shiny_access_token=NULL, ...){
   formals(f) <- c(formals(f), list(shiny_access_token=shiny_access_token))
 
   f(...)
-}
\ No newline at end of file
+}
